/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.sanskritlibrary.sanskritalignment;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.CatalogRecord;
import org.sanskritlibrary.Config;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.catalogs.AnnotationInfo;
import org.sanskritlibrary.catalogs.Annotations;
import org.sanskritlibrary.catalogs.Catalog;
import org.sanskritlibrary.catalogs.Pages;
import org.sanskritlibrary.catalogs.TextBlock;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;
import org.sanskritlibrary.catalogs.LocationInfo;


/**
 *
 * @author ralph
 */
public class GenerateDistributions extends SanskritLibraryServlet {
    String datadir;
    Regex reFolio = new Regex("", "(f\\.\\s*\\d+.*?[vr])");
                
    
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        int debug = 0;
        try {
            StringBuilder sbCharacterCounts = new StringBuilder();
            StringBuilder sbMinMaxInfo = new StringBuilder();
            ArrayList<ArrayList<ManuscriptSummary>> results = new ArrayList<ArrayList<ManuscriptSummary>>();
            datadir = getServletContext().getInitParameter("dataDirectory");
            if (!datadir.endsWith("/")) {
                datadir += "/";
            }
            String configFile = datadir + "config.xml";
            
            Config config = new Config(configFile);

            Pages pages = new Pages(config.imagesLayout);
            Catalog catalog = new Catalog(config.masterCatalog, pages, "slp");
            ArrayList<CatalogRecord> catalogs = catalog.catalogs;
            ArrayList<String> manuscripts = new ArrayList<String>();
            sbCharacterCounts.append("[\n");
            sbMinMaxInfo.append("[\n");
            for (CatalogRecord cr : catalogs) {
                debug++;
                if (debug == 135) {
                    debug = 135;
                }
                if (cr.catalogName.equals("UPenn Ms. Coll. 390, Item 375")) {
                        System.out.println("here");
                }
                results.clear();
                if (deliverData(config, request, response, cr, results)) {
                    manuscripts.add(cr.catalogName);
                    sbCharacterCounts.append(String.format("/*%s*/ [", cr.catalogName));
                    boolean firstText = true;
                    int max = 0;
                    int min = Integer.MAX_VALUE;
                    String minPage = null;
                    String maxPage = null;
                    
                    for (ArrayList<ManuscriptSummary> mss : results) {      // for each critical text referenced by the manuscript
                        if (mss.size() > 0) {
                            for (ManuscriptSummary ms : mss) {
                                sbCharacterCounts.append(String.format("%d,", ms.getCharacterCount()));
                            }
                        
                            int maxIdx = findmax(mss, firstText, max);
                            if (mss.get(maxIdx).count > max) {
                                maxPage = mss.get(maxIdx).filename;
                                max = mss.get(maxIdx).count;
                            }

                            int minIdx = findmin(mss, firstText, min);
                            if (mss.get(minIdx).count < min) {
                                minPage = mss.get(minIdx).filename;
                                max = mss.get(minIdx).count;
                            }
                            firstText = false;
                        }
                    }
                    
                    sbCharacterCounts.append("],\n");
                    
                    sbMinMaxInfo.append(String.format("{minPage:'%s', maxPage:'%s'}, // %s\n", 
                            minPage, maxPage, cr.catalogName));                    
                }
            }
            sbCharacterCounts.append("]");
            sbMinMaxInfo.append("]");
            
            StringBuilder sbManuscripts = new StringBuilder();
            sbManuscripts.append("[");
            for (String name : manuscripts) {
                sbManuscripts.append(String.format("'%s',\n", name));
            }
            sbManuscripts.append("]");
                        
            String template = IOAssistance.readEntireFile(this.getServletContext().getRealPath("templateDistributions.txt"));

            String page = String.format(template, sbManuscripts, sbCharacterCounts, sbMinMaxInfo);
            
            response.setContentType("text/html");
            response.setCharacterEncoding("UTF-8");
            PrintWriter pw = response.getWriter();
            pw.write(page);
            String test = this.getServletContext().getRealPath("test.html");
            BufferedWriter bw = IOAssistance.openWriter(test);
            bw.write(page);
            bw.close();
        } catch (Exception ex) {
            throw new IOException(String.format("debug=%d, %s", debug, ex.getMessage()));
        }
    }

    private int findmax(ArrayList<ManuscriptSummary> ms, boolean firstText, int sofar) {
        int max, len, count, maxIdx = 0;
        len = ms.size();
        if (firstText) {
            max = ms.get(0).getCharacterCount();
        } else {
            max =sofar;
        }
        for (int i=1; i<len; i++) {
            count = ms.get(i).getCharacterCount();
            if (count > max) {
                max = count;
                maxIdx = i;
            }
        }
        return maxIdx;
    }

    private int findmin(ArrayList<ManuscriptSummary> ms, boolean firstText, int sofar) {
        int min, len, count, minIdx = 0;
        len = ms.size();
        if (firstText) {
            min = ms.get(0).getCharacterCount();        
        } else {
            min = sofar;
        }
        for (int i=1; i<len; i++) {
            count = ms.get(i).getCharacterCount();
            if (count < min) {
                min = count;
                minIdx = i;
            }
        }
        return minIdx;
    }

    private boolean deliverData(Config config, HttpServletRequest request, 
            HttpServletResponse response, CatalogRecord cr,
            ArrayList<ArrayList<ManuscriptSummary>> results) 
            throws Exception {
        BufferedWriter bw = null;
        BufferedReader br = null;
        ArrayList<LocationInfo> locations = null;
        
        Hashtable<String, Integer> hashFolio = new Hashtable<String, Integer>();
        locations = new ArrayList<LocationInfo>();
        for (String textName : cr.textTypes) {
            String annotationsDirectory = String.format("%s%s/%s",
                    config.imageAnnotations, textName, cr.directoryName);

            File dir = new File(annotationsDirectory);
            File[] files = dir.listFiles();
            if (files == null) {
                System.out.format("%s not found\n", annotationsDirectory);
                return false;
            }
            for (File file : files) {
                String fn = file.getName();
                if (fn.endsWith(".xml")) {
                    hashFolio.clear();
                    Annotations pageAnnotations = new Annotations();
                    String annotationDirectory = String.format("%s%s/%s", config.imageAnnotations, textName, cr.directoryName);
                    pageAnnotations.loadAnnotations(annotationDirectory, annotationDirectory + "/" +fn);
                    for (AnnotationInfo ai : pageAnnotations.getAnnotations()) {
                        ArrayList<TextBlock> tms = ai.getTextMarkers();
                        if (tms.isEmpty()) {
                            continue;       // not in critical edition
                        }
                        if (tms.size() > 1) {
                            throw new Exception("Number of text blocks does not equal 1");
                        }
                        TextBlock tb = tms.get(0);
                        String comment = ai.getComment();
                        reFolio.reset(comment); 
                        if (reFolio.find()) {
                            String key = reFolio.getGroup(1);
                            Integer lno = hashFolio.get(key);
                            if (lno == null) {
                                hashFolio.put(key, locations.size());
                                locations.add(new LocationInfo(fn, tb.startRow, tb.endRow, key));                                
                            } else {
                                LocationInfo li = locations.get(lno);
                                li.totalChars += tb.endRow - tb.startRow + 1;
                            }
                        }
                    }
                }
            }
            // reach here have all locations for all annotations for a particular manuscript, e.g. 2252
                                    
            ArrayList<ManuscriptSummary> a = new ArrayList<ManuscriptSummary>();
            results.add(a);
            for (LocationInfo li : locations) {
                a.add(new ManuscriptSummary(String.format("%s[%s]", li.filename, li.comment), li.totalChars));
            }            
        }
        
        return true;
    }
        
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

    private class ManuscriptSummary {
        private String filename;
        private int count;
        
        ManuscriptSummary(String filename, int characterCount) {
            this.filename = filename;
            if (characterCount < 0) {
                characterCount = -characterCount;
            }
            this.count = characterCount;
        }
        
        int getCharacterCount() {
            return count;
        }
    }
}
