/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.sanskritlibrary.sanskritalignment;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.sanskritlibrary.CatalogRecord;
import org.sanskritlibrary.Config;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.catalogs.AnnotationInfo;
import org.sanskritlibrary.catalogs.Annotations;
import org.sanskritlibrary.catalogs.Catalog;
import org.sanskritlibrary.catalogs.LocationInfo;
import org.sanskritlibrary.catalogs.Pages;
import org.sanskritlibrary.catalogs.TextBlock;
import org.sanskritlibrary.servlet.SanskritLibraryServlet;

/**
 *
 * @author ralph
 */
public class GetAlignmentData extends SanskritLibraryServlet {
    String datadir;
    
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        int seqno = Integer.parseInt(request.getParameter("seqno"));
        try {
            datadir = getServletContext().getInitParameter("dataDirectory");
            if (!datadir.endsWith("/")) {
                datadir += "/";
            }
            String configFile = datadir + "config.xml";
            
            Config config = new Config(configFile);

            Pages pages = new Pages(config.imagesLayout);
            Catalog catalog = new Catalog(config.masterCatalog, pages, "slp");
            ArrayList<CatalogRecord> catalogs = catalog.catalogs;
            for (CatalogRecord cr : catalogs) {
                if (cr.seqno == seqno) {
                    deliverData(config, request, response, cr);
                    break;
                }
            }
        } catch (Exception ex) {
            throw new IOException(ex);
        }
    }

    private void deliverData(Config config, HttpServletRequest request, HttpServletResponse response, CatalogRecord cr) 
            throws Exception {
        BufferedWriter bw = null;
        BufferedReader br = null;
        StringBuilder sbImageLinks = new StringBuilder();
        ArrayList<String> criticalEditionCompressed = new ArrayList<String>();
        StringBuilder sbCriticalEdition = new StringBuilder();
        StringBuilder sbCriticalEditionCompressed = new StringBuilder();
        StringBuilder sbLocations = new StringBuilder();
        ArrayList<LocationInfo> locations = null;
        
        Regex reFolio = new Regex("", "f\\.\\s*\\d+.*[vr]$");
        sbImageLinks.append("[\n");
        
        ArrayList<String> pages = cr.getPages();
        for (String page : pages) {
            sbImageLinks.append(String.format("'%s',\n", page));
        }
        sbImageLinks.append("];");
        
        String text = null;


        
        sbCriticalEdition.append("[\n");
        for (String textName : cr.textTypes) {
            String fn = String.format("%stexts/%s.slp", datadir, textName);
            text = IOAssistance.readEntireFile(fn);
            String[] lines = text.split("\n");
            for (int i=0; i<4620; i++) {
                String line = lines[i];
                sbCriticalEdition.append(String.format("\"%s\",\n", line));
            }
        }
        sbCriticalEdition.append("];\n");

        
        ArrayList<String[]> allLinesDeva = new ArrayList<String[]>();
        for (String textName : cr.textTypes) {
            String fn = String.format("%stexts/%s.deva", datadir, textName);
            text = IOAssistance.readEntireFile(fn);
            String[] lines = text.split("\n");
            allLinesDeva.add(lines);
        }

        
        int i = 0;
        Hashtable<String, Integer> hashFolio = new Hashtable<String, Integer>();
        for (String textName : cr.textTypes) {
            String[] criticalEdition = allLinesDeva.get(i++); 
            locations = new ArrayList<LocationInfo>();
            String annotationsDirectory = String.format("%s%s/%s",
                    config.imageAnnotations, textName, cr.directoryName);

            File dir = new File(annotationsDirectory);
            File[] files = dir.listFiles();
            for (File file : files) {
                String fn = file.getName();
                if (fn.endsWith(".xml")) {
                    hashFolio.clear();
                    Annotations pageAnnotations = new Annotations();
                    String annotationDirectory = String.format("%s%s/%s", config.imageAnnotations, textName, cr.directoryName);
                    pageAnnotations.loadAnnotations(annotationDirectory, annotationDirectory + "/" +fn);
                    int prevEnd = 0;
                    for (AnnotationInfo ai : pageAnnotations.getAnnotations()) {
                        ArrayList<TextBlock> tms = ai.getTextMarkers();
                        if (tms.isEmpty()) {
                            continue;       // not in critical edition
                        }
                        if (tms.size() > 1) {
                            throw new Exception("Number of text blocks does not equal 1");
                        }
                        TextBlock tb = tms.get(0);
                        if (tb.startRow < prevEnd) {
                            //throw new Exception("Annotations out of order");
                            //continue;
                        }
                        /*
                        sb.append(String.format("{start:%d, end:%d},\n",
                                tb.startRow, tb.endRow));
                        */
                        prevEnd = tb.endRow;
                        String comment = ai.getComment();
                        reFolio.reset(comment); 
                        if (reFolio.find()) {
                            Integer lno = hashFolio.get(comment);
                            if (lno == null) {
                                hashFolio.put(comment, locations.size());
                                locations.add(new LocationInfo(fn, tb.startRow, tb.endRow, comment));                                
                            } else {
                                LocationInfo li = locations.get(lno);
                                li.endOffset = tb.endRow;
                            }
                        }
                    }
                }
            }
            // reach here have all locations for all annotations for a particular manuscript, e.g. 2252
                        
            int offset = 0;
            int n = 0;
            int currentLineNumber = -1;
            LocationInfo currentLocation = locations.get(n);
            for (String line : criticalEdition) {
                currentLineNumber++;
                int offsetEnd = offset + line.length(); // this moves past eol character
                if (currentLocation.startOffset >= offset && currentLocation.startOffset <= offsetEnd) {
                    currentLocation.startRow = currentLineNumber + 1;
                    n++;
                    if (n == locations.size()) {
                        break;
                    } else {
                        currentLocation = locations.get(n);
                    }
                }
                //System.out.format("%d - %d\n", offset, offsetEnd);
                offset = offsetEnd+1;     // offset of next line of critical edition
            }
            
            bw = IOAssistance.openWriter(this.getServletContext().getRealPath("dump.txt"));
            n = 1;
            int total = 0;
            for (LocationInfo li : locations) {
                bw.write(String.format("%d. ", n++));
                bw.write(li.toString());
                bw.write("\n");
                bw.write("*** ");
                bw.write(criticalEdition[li.startRow-1]);
                bw.write("\n");
                bw.write(text.substring(li.startOffset, li.endOffset+1));
                //System.out.println(li.endOffset - li.startOffset + 1);
                //total += li.endOffset - li.startOffset + 1;
            }
            //System.out.println(total);
            bw.close();
        }      
        
        response.setContentType("text/html");
        response.setCharacterEncoding("UTF-8");
        PrintWriter pw = response.getWriter();
        String template = IOAssistance.readEntireFile(this.getServletContext().getRealPath("template.txt"));
        String fn2 = String.format("%stexts/%s.slp", datadir, "BhP10");
        int charcntPerPageEstimate = estimateCharactersPerPage(fn2, 0, 4620, 290, 
                sbCriticalEditionCompressed, criticalEditionCompressed);
        estimateRowNumbers(locations, criticalEditionCompressed, charcntPerPageEstimate, 0, 4620);
        
        sbLocations.append("[");
        for (LocationInfo li : locations) {
            sbLocations.append(String.format("{filename:'%s', startRow:%d, diff:%d, estimatedStartLine:%d},\n",
                        li.filename,
                        li.startRow,
                        li.diff,
                        li.estimatedStartLine));
        }
        sbLocations.append("];");

        String page = String.format(template, 
                cr.textTypes.get(0), 
                sbImageLinks.toString(),
                sbCriticalEdition.toString(),
                sbCriticalEditionCompressed.toString(),
                sbLocations.toString(),
                charcntPerPageEstimate);

        pw.write(page);

        int total = 0;
        for (int k=0; k<locations.size(); k++) {
            LocationInfo li = locations.get(k);
            total += li.endOffset - li.startOffset+1;
            //System.out.format("%s: %d (%d~=%d)\n", li.filename, li.endOffset - li.startOffset+1, total, charcntPerPageEstimate*(i+1));
        }
    }
    
    void estimateRowNumbers(ArrayList<LocationInfo> locations, ArrayList<String> criticalEditionCompressed,
                    int charcntPerPageEstimate, int first, int last) {
        int numpages = locations.size();
        locations.get(0).diff = 0;
        int numlines = criticalEditionCompressed.size();
        int estimatedStartLine = 0;
        int fudge = 0;
        int j = 0;
        for (int i=0; i < numpages; i++) {
            LocationInfo location = locations.get(i);
            location.estimatedStartLine = estimatedStartLine;
            location.diff = location.startRow-1 - estimatedStartLine;            
            int totalChars = 0;
            while (true) {
                if (fudge == 0) {
                    totalChars += criticalEditionCompressed.get(j).length();
                } else {
                    totalChars += fudge;        // count remaining characters on last line
                    fudge = 0;
                }
                if (totalChars == charcntPerPageEstimate) {
                    estimatedStartLine = j+1;
                    break;
                } else if (totalChars > charcntPerPageEstimate) {
                    estimatedStartLine = j;     
                    fudge = totalChars - charcntPerPageEstimate;
                    break;
                } else {
                    j++;
                }
            }
        }
    }
    
    int estimateCharactersPerPage(String fn, int first, int last, int numPages, StringBuilder sb, 
            ArrayList<String> criticalEditionCompressed) throws Exception {
        int estimate = 0;
        String text = IOAssistance.readEntireFile(fn);
        String[] lines = text.split("\n");
        sb.append("[\n");
        for (int i=first; i<last; i++) {
            String line = lines[i];
            line = line.replaceAll("^[0-9. ]*", "");
            line = line.replaceAll("\\d+(\\d\\d\\d)\\s*\\.\\.\\s*$", "xx ..");
            line = line.replaceAll("[\\s-]", "");      
            criticalEditionCompressed.add(line);
            sb.append(String.format("\"%s\",\n", line));
            estimate += line.length();
        }
        sb.append("]");
        int n = estimate/numPages;
        System.out.println(estimate - n * numPages);
        return n;
    }
    
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
    
}
