"use strict";
$(function() {
    var btn, stats, width, height, leftMargin, rightMargin, maxHeight, catalogNumber;

    catalogNumber = 0;
    maxHeight = 600;
    leftMargin = 0;
    rightMargin = 0;
    
    btn = $("#btnNext");
    btn.click(next);
    
    btn = $("#btnPrev");
    btn.click(prev);
    
    stats = $("#stats");
    
    prepare();

	function debug(a) {
		var i, len, countAbove, countBelow, countEqual, avg, min;
		len = a.length;
		countAbove = 0;
		countBelow = 0;
		countEqual = 0;
		avg = computeAverage(a, 1, 0);
		for (i=0; i<len; i++) {
			if (a[i] < avg) {
				countBelow++;
			} else if (a[i] > avg) {
				countAbove++;
			} else {
				countEqual++;
			}
		}
		console.log("below=" + countBelow + ", above=" + countAbove + ", equal=" + countEqual);
	}
    function prepare() {
        var div, p, canvas, currentManuscript, ctx, max, min, adjust, avg, obj, minIndex, maxIndex;
        
        div = $("#graphs");
        div.empty();
        p = $("<p>");
        div.append(p);
        obj = minmaxInfo[catalogNumber];
        p.html(manuscripts[catalogNumber] + "<br/>minPage=" + obj.minPage + "<br/>maxPage=" + obj.maxPage);

        currentManuscript = manuscriptInfo[catalogNumber];
        fixup(currentManuscript);
        debug(currentManuscript);
        width = currentManuscript.length * 10;
        if (width > 0) {
            canvas = $("<canvas>");
            div.append(canvas);
            obj = findmax(currentManuscript);
            max = obj.value;
            maxIndex = obj.index;
            
            obj = findmin(currentManuscript);
            min = obj.value;
            minIndex = obj.index;
            
            avg = computeAverage(currentManuscript, 1, 0);
            stats.html("min=" + min + ", max=" + max + ", avg=" + Math.round(avg) + ", max-avg=" + Math.round(max-avg) + ", avg-min=" + Math.round(avg-min));
            if (max - min > maxHeight) {
                adjust = maxHeight / (max - min);
                height = maxHeight;
            } else {
                adjust = 1;
                height = max - min + 1;
                //height = maxHeight;
            }

            canvas.attr("width", "" + width + "px");
            canvas.attr("height", "" + height + "px");
            ctx = canvas[0].getContext('2d');
            drawGraph(ctx, currentManuscript, adjust, min, minIndex, maxIndex);
        } else {
            p = $("<p>");
            div.append(p);
            p.css("color", "red");
            p.html("No data");
        }
    }
    



    function drawGraph(ctx, values, adjust, min, minIndex, maxIndex) {
        var x, y, w, h, numpages, i, yMiddle, prevX, prevY, a, xmin, xmax;
        ctx.beginPath();            // crucial to do this or lines won't be cleared!
        ctx.fillStyle = '#ffffff';
        ctx.fillRect(0, 0, width, height);

        // draw X axis
        
        ctx.lineWidth = 2;
        yMiddle = height - computeAverage(values, adjust, min);
        ctx.moveTo(leftMargin, yMiddle);
        ctx.lineTo(width - rightMargin, yMiddle);
        ctx.stroke();
        w = 2;
        h = 2;

        // plot points
        prevX = -1;
        prevY = -1;
        numpages = values.length;
		a = [];
        for (i=0; i < numpages; i++) {
            x = leftMargin + i*10;
            y = height-adjust*(values[i] - min);
            a.push(y);
            if (i === minIndex) {
                ctx.strokeStyle = '#00ff00';
            } else if (i === maxIndex) {
                ctx.strokeStyle = '#0000ff';
            } else {
		        ctx.strokeStyle = '#000000';
            }
            ctx.fillRect(x, y, w, h);
            if (prevX !== -1) {
                ctx.beginPath();
                ctx.moveTo(prevX, prevY);
                ctx.lineTo(x, y);
                ctx.stroke();
            }
            prevX = x;
            prevY = y;
        }
        debug(values);
        debug(a);
    }    

    
    function prev() {
        if (catalogNumber === 0) {
            alert("You are at the first manuscript");
        } else {
            catalogNumber--;
            prepare();
        }
    }
    
    function next() {
        if (catalogNumber + 1 === manuscriptInfo.length) {
            alert("You have seen all the manuscripts");
        } else {
            catalogNumber++;
            prepare();
        }
    }

    function findmax(a) {
        var max, len, i, idx;
        
        max = 0;
        len = a.length;
        for (i=0; i < len; i++) {
            if (a[i] > max) {
                max = a[i];
                idx = i;
            }
        }
        
        return {value:max, index:idx};
    }
    
	
	function fixup(a) {
        var len, i;
        
        len = a.length;
        for (i=0; i < len; i++) {
            if (a[i] < 0) {
                a[i] = -a[i];
            }
        }
	}

    function findmin(a) {
        var min, len, i, idx;
        
        min = a[0];
        len = a.length;
        for (i=1; i < len; i++) {
            if (a[i] < min) {
            	idx = i;
                min = a[i];
            }
        }
        
        return {value:min, index:idx};
    }
    
    function computeAverage(a, adjust, min) {
        var i, len, total;
        
        total = 0;
        len = a.length;
        for (i=0; i<len; i++) {
            total += a[i] - min;
        }
        
        return (total/len) * adjust;
    }

});

